/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.sample

import android.os.Bundle
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.friend.client.PickerClient
import com.kakao.sdk.friend.model.*
import com.kakao.sdk.partner.friend.client.selectFriend
import com.kakao.sdk.partner.friend.client.selectFriendPopup
import com.kakao.sdk.partner.friend.client.selectFriends
import com.kakao.sdk.partner.friend.client.selectFriendsPopup
import com.kakao.sdk.partner.friend.model.PickerFriendRequestParams
import com.kakao.sdk.partner.sample.databinding.ActivityFriendPickerSettingBinding

class FriendPickerSettingActivity : AppCompatActivity() {
    private lateinit var binding: ActivityFriendPickerSettingBinding
    private var title: String? = null
    private var serviceTypeFilter: PickerServiceTypeFilter? = null
    private var friendFilter: PickerFriendFilter? = null
    private var viewAppearance: ViewAppearance? = null
    private var countryCodes: List<String>? = null
    private var osFilter: PickerUsingOsFilter? = null
    private var orientation: PickerOrientation? = null
    private var disableSelectOptions: List<DisableSelectOption>? = null
    private var maxPickableCount: Int = 30
    private var minPickableCount: Int = 1

    private val disableCustoms = mutableListOf<DisableSelectOption>()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityFriendPickerSettingBinding.inflate(layoutInflater)
        setContentView(binding.root)

        val friendTypeArray = resources.getStringArray(R.array.friend_type)
        val friendFilterArray = resources.getStringArray(R.array.friend_filter)
        val deviceOsArray = resources.getStringArray(R.array.device_os)
        val orientationArray = resources.getStringArray(R.array.picker_orientation)
        val viewAppearanceArray = resources.getStringArray(R.array.view_appearance)

        binding.apply {
            disableSelectClearBtn.setOnClickListener {
                disableCustoms.clear()
                disableCustomCount.text = "${disableCustoms.size}"
            }

            disableSelectAddBtn.setOnClickListener {
                val message = disableMessageEt.text.toString()
                val uuidText = uuidEt.text.toString()
                val uuids = if (uuidText.isEmpty()) {
                    null
                } else {
                    uuidText.split(",").map { it.trim() }.toList()
                }
                disableCustoms.add(DisableSelectOption(DisableSelectReason.CUSTOM, message, uuids))

                disableMessageEt.text.clear()
                uuidEt.text.clear()
                disableCustomCount.text = "${disableCustoms.size}"
            }

            cancelBtn.setOnClickListener {
                finish()
            }

            createBtn.setOnClickListener {
                title = getPickerTitle()
                maxPickableCount = getMaxPickableCount()
                minPickableCount = getMinPickableCount()
                countryCodes = getCountryCodes()
                serviceTypeFilter = getPickerFriendType(friendTypeArray)
                friendFilter = getPickerFriendFilter(friendFilterArray)
                osFilter = getOsFilter(deviceOsArray)
                viewAppearance = getViewAppearance(viewAppearanceArray)
                orientation = getPickerOrientation(orientationArray)
                disableSelectOptions = getDisableSelect()

                val callback = { selectedUsers: SelectedUsers?, error: Throwable? ->
                    if (error != null) {
                        SdkLog.e("친구 정보 불러오기 실패 $error")
                        Toast.makeText(
                            this@FriendPickerSettingActivity,
                            "친구 정보 불러오기 실패 $error",
                            Toast.LENGTH_LONG
                        ).show()
                    } else {
                        SdkLog.i("친구 정보 불러오기 성공 $selectedUsers")
                        Toast.makeText(
                            this@FriendPickerSettingActivity,
                            "친구 정보 불러오기 성공 $selectedUsers",
                            Toast.LENGTH_LONG
                        ).show()
                    }
                }

                val param = getPickerRequestParams()
                if (popupCb.isChecked) {
                    if (singleCb.isChecked) {
                        PickerClient.instance.selectFriendPopup(
                            this@FriendPickerSettingActivity,
                            param,
                            callback
                        )
                    } else {
                        PickerClient.instance.selectFriendsPopup(
                            this@FriendPickerSettingActivity,
                            param,
                            callback
                        )
                    }
                } else {
                    if (singleCb.isChecked) {
                        PickerClient.instance.selectFriend(
                            this@FriendPickerSettingActivity,
                            param,
                            callback
                        )
                    } else {
                        PickerClient.instance.selectFriends(
                            this@FriendPickerSettingActivity,
                            param,
                            callback
                        )
                    }
                }
            }
        }
    }

    private fun ActivityFriendPickerSettingBinding.getDisableSelect(): List<DisableSelectOption>? {
        val filter = mutableListOf<DisableSelectOption>()
        if (binding.msgBlockedCb.isChecked) filter.add(DisableSelectOption(DisableSelectReason.MSG_BLOCKED))
        if (binding.registeredCb.isChecked) filter.add(DisableSelectOption(DisableSelectReason.REGISTERED))
        if (binding.unregisteredCb.isChecked) filter.add(DisableSelectOption(DisableSelectReason.UNREGISTERED))
        filter.addAll(disableCustoms)
        return if (filter.isEmpty()) null else filter
    }

    private fun ActivityFriendPickerSettingBinding.getPickerOrientation(
        orientationArray: Array<String>
    ) = when (orientationSpinner.selectedItem) {
        orientationArray[1] -> PickerOrientation.AUTO
        orientationArray[2] -> PickerOrientation.PORTRAIT
        orientationArray[3] -> PickerOrientation.LANDSCAPE
        else -> null
    }

    private fun ActivityFriendPickerSettingBinding.getViewAppearance(
        viewAppearanceArray: Array<String>
    ) = when (themeSpinner.selectedItem) {
        viewAppearanceArray[1] -> ViewAppearance.AUTO
        viewAppearanceArray[2] -> ViewAppearance.LIGHT
        viewAppearanceArray[3] -> ViewAppearance.DARK
        else -> null
    }

    private fun ActivityFriendPickerSettingBinding.getOsFilter(
        deviceOsArray: Array<String>
    ) = when (osSpinner.selectedItem) {
        deviceOsArray[1] -> PickerUsingOsFilter.ALL
        deviceOsArray[2] -> PickerUsingOsFilter.ANDROID
        deviceOsArray[3] -> PickerUsingOsFilter.IOS
        else -> null
    }

    private fun ActivityFriendPickerSettingBinding.getPickerFriendFilter(
        friendFilterArray: Array<String>
    ) = when (friendFilterSpinner.selectedItem) {
        friendFilterArray[1] -> PickerFriendFilter.REGISTERED
        friendFilterArray[2] -> PickerFriendFilter.INVITABLE
        friendFilterArray[3] -> PickerFriendFilter.NONE
        else -> null
    }

    private fun ActivityFriendPickerSettingBinding.getPickerFriendType(
        friendTypeArray: Array<String>
    ) = when (friendTypeSpinner.selectedItem) {
        friendTypeArray[1] -> PickerServiceTypeFilter.TALK
        friendTypeArray[2] -> PickerServiceTypeFilter.STORY
        friendTypeArray[3] -> PickerServiceTypeFilter.TALKSTORY
        else -> null
    }

    private fun ActivityFriendPickerSettingBinding.getCountryCodes() =
        if (countryCodeEt.text.isNullOrBlank()) {
            null
        } else {
            countryCodeEt.text.toString().split(',').toList()
        }

    private fun ActivityFriendPickerSettingBinding.getMinPickableCount() =
        if (minCountEt.text.toString().isBlank()) {
            minCountEt.hint.toString().toInt()
        } else {
            minCountEt.text.toString().toInt()
        }

    private fun ActivityFriendPickerSettingBinding.getMaxPickableCount() =
        if (maxCountEt.text.toString().isBlank()) {
            maxCountEt.hint.toString().toInt()
        } else {
            maxCountEt.text.toString().toInt()
        }

    private fun ActivityFriendPickerSettingBinding.getPickerTitle() =
        if (titleEt.text.toString().isEmpty()) {
            null
        } else {
            titleEt.text.toString()
        }

    private fun getPickerRequestParams(): PickerFriendRequestParams {
        return binding.run {
            PickerFriendRequestParams(
                title = title,
                serviceTypeFilter = serviceTypeFilter,
                friendFilter = friendFilter,
                countryCodeFilters = countryCodes,
                viewAppearance = viewAppearance,
                usingOsFilter = osFilter,
                orientation = orientation,
                enableSearch = searchbarCb.isChecked,
                enableIndex = indexCb.isChecked,
                showMyProfile = myProfileCb.isChecked,
                showFavorite = myFavoriteCb.isChecked,
                disableSelectOptions = disableSelectOptions,
                maxPickableCount = maxPickableCount,
                minPickableCount = minPickableCount,
                showPickedFriend = pickedCb.isChecked,
                displayAllProfile = allProfileCb.isChecked
            )
        }
    }

    companion object {
        const val TAG = "KakaoSDKSample"
    }
}